/*
 * Copyright (C) 2004,2005  Heinz Mauelshagen, Red Hat GmbH.
 *                          All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#ifndef _LIST_H
#define _LIST_H

/*
 * Double-linked list definitons and macros.
 */

struct list_head {
	struct list_head *next;
	struct list_head *prev;
};

#define LIST_HEAD_INIT(a)	struct list_head a = { &(a), &(a) }
#define INIT_LIST_HEAD(a)	do { (a)->next = (a)->prev = (a); } while(0)

#define list_empty(list)	((list)->next == (list))

static inline void __list_add(struct list_head *new,
			      struct list_head *prev,
			      struct list_head *next)
{
	next->prev = new;
	new->next = next;
	new->prev = prev;
	prev->next = new;
}

#define list_add(new, pos)	__list_add((new), pos, (pos)->next)
#define list_add_tail(new, pos)	__list_add((new), (pos)->prev, pos)

#define	list_del(pos) { \
	(pos)->next->prev = (pos)->prev; \
	(pos)->prev->next = (pos)->next; \
	(pos)->next = (pos)->prev = 0; \
}

#define list_entry(pos, type, member) \
	((type*)((char*)(pos)-(unsigned long)(&((type*)0)->member)))

#define list_for_each(pos, head) \
	for (pos = (head)->next; pos != head; pos = (pos)->next)

#define list_for_each_entry(pos, head, member) \
        for (pos = list_entry((head)->next, typeof(*pos), member); \
             &(pos)->member != (head); \
             pos = list_entry((pos)->member.next, typeof(*pos), member))

#define list_for_each_safe(pos, n, head) \
        for (pos = (head)->next, n = pos->next; \
             pos != (head); \
             pos = n, n = pos->next)

#endif
