/*
 * Copyright (C) 2004,2005  Heinz Mauelshagen, Red Hat GmbH.
 *                          All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#ifndef _LIB_CONTEXT_H_
#define _LIB_CONTEXT_H_

#include "datastruct/list.h"
#include "locking/locking.h"
#include "misc/misc.h"

enum lc_lists {
	LC_FORMATS = 0,	/* Metadata format handlers. */
	LC_DISK_INFOS,	/* Disks discovered. */
	LC_RAID_DEVS,	/* Raid devices discovered. */
	LC_RAID_SETS,	/* Raid sets grouped. */
	/* Add new lists below here ! */
	LC_LISTS_SIZE,	/* Must be the last enumerator. */
};

/* List access macros. */
#define	LC_FMT(lc)	(lc_list((lc), LC_FORMATS))
#define	LC_DI(lc)	(lc_list((lc), LC_DISK_INFOS))
#define	LC_RD(lc)	(lc_list((lc), LC_RAID_DEVS))
#define	LC_RS(lc)	(lc_list((lc), LC_RAID_SETS))

enum lc_options {
	LC_COLUMN = 0,
	LC_DEBUG,
	LC_DUMP,
	LC_GROUP,
	LC_SETS,
	LC_TEST,
	LC_VERBOSE,
	LC_IGNORELOCKING,/* Add new options below this one ! */
	LC_OPTIONS_SIZE,/* Must be the last enumerator. */
};

struct lib_context {
	struct lib_version version;
	char *cmd;

	/* Option counters used throughout the library. */
	unsigned int opts[LC_OPTIONS_SIZE];

	/*
	 * Lists for:
	 *
	 *	o metadata format handlers the library supports
	 * 	o block devices discovered
	 * 	o RAID devices discovered
	 * 	o RAID sets grouped
	 */
	struct list_head lists[LC_LISTS_SIZE];

	char *locking_name;		/* Locking mechanism selector. */
	struct	locking *lock;		/* Resource locking. */

	mode_t	mode;			/* File/directrory create modes. */

	struct {
		const char *error;	/* For error mappings. */
	} path;

	/* Monitoring daemon: */
	const char *daemon_interface_name;/* Monitoring daemon interface
					     mechanism selector. */
	struct daemon_interface *daemon; /* Daemon interface. */
};

struct lib_context *alloc_lib_context(char **argv);
void free_lib_context(struct lib_context *lc);
int lc_opt(struct lib_context *lc, enum lc_options o);
int lc_inc_opt(struct lib_context *lc, int o);
struct list_head *lc_list(struct lib_context *lc, int l);

#endif
