/*
 * Copyright (C) 2004,2005  Heinz Mauelshagen, Red Hat GmbH.
 *                          All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

#ifndef _MISC_H_
#define _MISC_H_

#define struct_offset(s, member) ((unsigned short) &((struct s *) 0)->member)

struct lib_version {
	const char *text;
	const char *date;
	struct {
		unsigned int major;
		unsigned int minor;
		unsigned int sub_minor;
		const char *suffix;
	} v;
};

void sysfs_workaround(struct lib_context *lc);
void mk_alpha(struct lib_context *lc, char *str, size_t len);
char *get_basename(struct lib_context *lc, char *str);
char *get_dirname(struct lib_context *lc, char *str);
char *remove_white_space(struct lib_context *lc, char *str, size_t len);
char *remove_delimiter(char *ptr, char c);
void add_delimiter(char **ptr, char c);

struct lib_context *libdmraid_init(int argc, char **argv);
void libdmraid_exit(struct lib_context *lc);
const char *libdmraid_date(struct lib_context *lc);
const char *libdmraid_version(struct lib_context *lc);

int mk_dir(struct lib_context *lc, const char *dir);

int read_file(struct lib_context *lc, const char *who, char *path, void *buffer,
	      size_t size, loff_t offset);
int write_file(struct lib_context *lc, const char *who, char *path,
	       void *buffer, size_t size, loff_t offset);

int yes_no_prompt(struct lib_context *lc, const char *prompt, ...);

void free_string(struct lib_context *lc, char **string);
int p_fmt(struct lib_context *lc, char **string, const char *fmt, ...);

static inline uint64_t round_down(uint64_t what, unsigned int by)
{
	return what & ~((uint64_t) by - 1);
}

static inline uint64_t round_up(uint64_t what, unsigned int by)
{
	uint64_t t = round_down(what, by);

	return t == what ? t : t + by;
}

static inline uint64_t div_up(uint64_t what, unsigned int by)
{
	return round_up(what, by) / by;
}

#endif
