/*
 * Copyright (C) 2004,2005  Heinz Mauelshagen, Red Hat GmbH.
 *                          All rights reserved.
 *
 * See file LICENSE at the top of this source tree for license information.
 */

/*
 * See commands.[ch] for the perform() function call abstraction below.
 */

#include <dmraid.h>
#include "commands.h"
#include "toollib.h"
#include "version.h"

int main(int argc, char **argv)
{
	int ret = 0;
	struct lib_context *lc;

	/* Initialize library (creates a context to use it). */
	if ((lc = libdmraid_init(argc, argv))) {
		/* Initialize the tool library. */
		if (toollib_init(lc)) {
			/*
			 * Parse command line arguments and run 'early'
			 * functions for options which set library context
			 * variables (eg, --debug).
			 */
			if (handle_args(lc, argc, &argv)) {
				if ((ret = init_locking(lc)))
					/* Perform actions for options. */
					ret = perform(lc, argv);
			}
		
			/* Clean up before exit... */
			toollib_exit(lc);
		}
	
		/* Cleanup the library context. */
		libdmraid_exit(lc);
	}

	/* Inverse return code for exit. */
	return (ret ? 0 : 1);
}
